#---------------------------------------------------------------
# Fax
#----------------------------------------------------------------

#
# Configurations

set fax(viewer) "viewfax -geometry +0+0"
set fax(numberlistfile) "$c(tkscandir)/faxnumber"
set fax(c) 0

#
# window_Fax and make_Fax

proc window_Fax { wn } { global fax c w l
    label $wn.ti -text "TkScan Fax Manager" -width 24 -pady 4 -relief ridge
    canvas $wn.sep -width 0 -height 5 -relief flat
    pack  $wn.ti $wn.sep -side top

    frame $wn.fr -bd 1 -relief groove
    frame $wn.fr.l
    pack  $wn.fr.l -side top -fill x
    label $wn.fr.l.l -text "Fax to" -width 10 -anchor w
    button $wn.fr.l.b -text "Number List" -command faxnumberWindow \
                                    -width 13 -padx 2 -pady 0 -relief ridge
    button $wn.fr.l.c -text "Clear" -width  5 -padx 0 -pady 0 -relief ridge \
           -command {set fax(number) "" ; set fax(recipient) ""}
    pack $wn.fr.l.l $wn.fr.l.b $wn.fr.l.c -side left
    frame $wn.fr.fn
    pack  $wn.fr.fn -side top -fill x
    label $wn.fr.fn.l -text "Number" -width 10 -anchor e
    entry $wn.fr.fn.e -textvariable fax(number) \
                      -width 20 -font $l(f1) -fg $l(c1)
    pack $wn.fr.fn.l $wn.fr.fn.e -side left
    frame $wn.fr.re
    pack  $wn.fr.re -side top -fill x
    label $wn.fr.re.l -text Recipient -width 10 -anchor e
    entry $wn.fr.re.e -textvariable fax(recipient) \
                      -width 20 -font $l(f1) -fg $l(c1)
    pack $wn.fr.re.l $wn.fr.re.e -side left

    frame $wn.pl -bd 1 -relief groove
    frame $wn.pl.a
    pack $wn.pl.a -side top -fill x
    label $wn.pl.a.l -text "Page List" -width 10 -anchor w
    pack $wn.pl.a.l -side left
    frame $wn.pl.b
    pack $wn.pl.b -side top -fill x
    set w(faxpagelist) $wn.pl.b.b
    listbox $w(faxpagelist) -width 19 -height 8 -font $l(f1) -fg blue \
            -yscrollcommand "$wn.pl.b.s set"
    pack $wn.pl.b.b -side left
    scrollbar $wn.pl.b.s -orient vertical -command "$w(faxpagelist) yview"
    pack $wn.pl.b.s -side left -fill y
    frame $wn.pl.b.f
    pack $wn.pl.b.f -side right
    bind $w(faxpagelist) <Double-Button-1> {
        if {[$w(faxpagelist) curselection] != {}} {
            set page $fax([$w(faxpagelist) curselection])
            exec sh -c "$fax(viewer) $c(tkscandir)/$page" &
        }
    }
    button $wn.pl.b.f.1 -text Attach -width 5 -pady 2 -command {
        attachWindow
    }
    button $wn.pl.b.f.2 -text Detach -width 5 -pady 2 -command {
        if {[set t [$w(faxpagelist) curselection]] != {}} {
            for {set i $t} {$i < [expr $fax(c)-1]} {incr i} {
                set fax($i) $fax([expr $i+1])
            }
            incr fax(c) -1
            update_faxpagelist
            if {$t < $fax(c)} { $w(faxpagelist) selection set $t
                       } else { $w(faxpagelist) selection set [expr $t-1] }
        }
    }
    button $wn.pl.b.f.3 -text Up     -width 5 -pady 2 -command {
        if {[set t [$w(faxpagelist) curselection]] != {} && $t > 0} {
            set u [expr $t-1]
            set tmp $fax($u)
            set fax($u) $fax($t)
            set fax($t) $tmp
            update_faxpagelist
            $w(faxpagelist) selection set $u
        }
    }
    button $wn.pl.b.f.4 -text Down   -width 5 -pady 2 -command {
        if {[set t [$w(faxpagelist) curselection]] != {}
            && $t < [expr $fax(c)-1]} {
            set u [expr $t+1]
            set tmp $fax($u)
            set fax($u) $fax($t)
            set fax($t) $tmp
            update_faxpagelist
            $w(faxpagelist) selection set $u
        }
    }
    button $wn.pl.b.f.5 -text View   -width 5 -pady 2 -command {
        if $fax(c) {
            for {set pages ""; set i 0} {$i < $fax(c)} {incr i} {
                append pages " $c(tkscandir)/$fax($i)"
            }
            exec sh -c "$fax(viewer) $pages" &
        }
    }
    pack $wn.pl.b.f.1 $wn.pl.b.f.2 $wn.pl.b.f.3 $wn.pl.b.f.4 \
         $wn.pl.b.f.5 -side top

    frame $wn.b
    button $wn.b.1 -text "SEND FAX" -width 12 -command sendFax
    pack $wn.b.1 -side left -expand yes -fill x

    pack $wn.fr $wn.pl $wn.b -side top -expand yes -fill x

    update_faxpagelist
    setquickmode Fax
    setquickpaper A4
}

proc make_Fax { prefix i } { global fax s
    set tag $prefix-$i
    pnmtofax $tag
    stat_text "Scanning $s(curscan) / $s(nrect) ... $tag.fax"
    set fax($fax(c)) "$tag.fax"
    incr fax(c)
    update_faxpagelist
    update
}

#
# Efax dependent procedures

proc pnmtofax { fn } { global c
    set if "$c(tkscandir)/$fn.pnm"
    set of "$c(tkscandir)/$fn.fax"
    exec sh -c "efix -n $of $if ; rm -f $if" &
}

proc sendFax {} { global fax c
    if {$fax(c) <= 0} { stat_text "sendfax: Nothing to send" ; return }
    if {$c(faxnumber) == {}} { stat_text "sendfax: Empty Fax Number" ; return }
    for {set pages "" ; set i 0} {$i < $fax(c)} {incr i} {
        append pages " $c(tkscandir)/$fax($i)"
    }
    exec sh -c "fax send $fax(number) $pages" &
}

#
# Subroutines

proc update_faxpagelist {} { global fax w
    $w(faxpagelist) delete 0 end
    for {set i 0} {$i < $fax(c)} {incr i} {$w(faxpagelist) insert end $fax($i)}
}

proc faxnumberWindow {} { global fax l w
    if [winfo exists .pbook] {return}
    toplevel .pbook
    wm title .pbook "TkScan: FaxNumberList"
    frame .pbook.1
    label .pbook.1.l -text "Fax Number List"
    pack  .pbook.1.l -side left
    frame .pbook.l
    set w(faxnumberlist) .pbook.l.b
    listbox $w(faxnumberlist) -yscroll ".pbook.l.s set" \
            -width 40 -height 8 -relief sunken -font $l(f1)
    scrollbar .pbook.l.s -command "$w(faxnumberlist) yview"
    pack $w(faxnumberlist) -side left -fill both -expand yes
    pack .pbook.l.s -side right -fill y
    frame .pbook.b -bd 1 -relief ridge
    bind $w(faxnumberlist) <Double-Button-1> select_faxnumber
    button .pbook.b.1 -text Close  -width 6 -pady 0 -command {
        save_faxnumberlist
        destroy .pbook
    }
    button .pbook.b.2 -text Select -width 6 -pady 0 -command select_faxnumber
    button .pbook.b.3 -text New    -width 6 -pady 0 -command {
        set fax(tmprecipient) ""
        set fax(tmpnumber) ""
        set fax(tmpinsert) end
        focus .pbook.i.1.e
    }
    button .pbook.b.4 -text Edit   -width 6 -pady 0 -command {
        if {[set i [$w(faxnumberlist) curselection]] != {}} {
           set line [$w(faxnumberlist) get $i]
           set x [string last : $line]
           set fax(tmprecipient) [string range $line 0 [expr $x-2]]
           set fax(tmpnumber) [string range $line [expr $x+2] end]
           set fax(tmpinsert) $i
           focus .pbook.i.1.e
        }
    }
    button .pbook.b.5 -text Remove -width 6 -pady 0 -command {
        if {[set i [$w(faxnumberlist) curselection]] != {}} {
            $w(faxnumberlist) delete $i
        }
    }
    pack .pbook.b.1 .pbook.b.2 .pbook.b.3 .pbook.b.4 .pbook.b.5 -side left
    frame .pbook.i -bd 1 -relief ridge
    frame .pbook.i.1
    frame .pbook.i.2
    pack  .pbook.i.1 .pbook.i.2 -side top -fill x
    label .pbook.i.1.l -text Name -anchor e -width 12
    label .pbook.i.2.l -text "Fax Number" -anchor e -width 12
    entry .pbook.i.1.e -textvariable fax(tmprecipient) -width 25 \
                       -font $l(f1) -fg $l(c1)
    entry .pbook.i.2.e -textvariable fax(tmpnumber) -width 25 \
                       -font $l(f1) -fg $l(c1)
    button .pbook.i.1.b -text "Clear" -width 6 -pady 0 -command {
        set fax(tmprecipient) ""
        set fax(tmpnumber) ""
    }
    button .pbook.i.2.b -text "Add" -width 6 -pady 0 -command {
        if {$fax(tmpnumber) != {}} {
            if ![string match end $fax(tmpinsert)] {
                $w(faxnumberlist) delete $fax(tmpinsert)
            }
            $w(faxnumberlist) insert $fax(tmpinsert) \
                         "$fax(tmprecipient) : $fax(tmpnumber)"
        }
    }
    bind .pbook.i.1.e <KeyPress-Return> {focus .pbook.i.2.e}
    bind .pbook.i.2.e <KeyPress-Return> {
        if ![string match end $fax(tmpinsert)] {
            $w(faxnumberlist) delete $fax(tmpinsert)
        }
        $w(faxnumberlist) insert $fax(tmpinsert) \
                         "$fax(tmprecipient) : $fax(tmpnumber)"
        focus .pbook.i.1.e
    }
    pack .pbook.i.1.l .pbook.i.1.e .pbook.i.1.b -side left
    pack .pbook.i.2.l .pbook.i.2.e .pbook.i.2.b -side left

    pack .pbook.1 .pbook.i .pbook.l .pbook.b -side top -fill x

    set fax(tmprecipient) ""
    set fax(tmpnumber) ""
    set fax(tmpinsert) end
    read_faxnumberlist
}

proc select_faxnumber {} { global fax w
    if {[set s [$w(faxnumberlist) curselection]] != {}} {
        set line [$w(faxnumberlist) get $s]
        set x [string last : $line]
        set fax(number) [string range $line [expr $x+2] end]
        set fax(recipient) [string range $line 0 [expr $x-2]]
    }
}

proc read_faxnumberlist {} { global fax w
    if ![file exists $fax(numberlistfile)] {return}
    set file [open $fax(numberlistfile) r]
    while {[set i [gets $file]]!={}} { $w(faxnumberlist) insert end $i }
    close $file
}

proc save_faxnumberlist {} { global fax w
    set file [open $fax(numberlistfile) w]
    for {set i 0} {$i < [$w(faxnumberlist) size]} {incr i} {
        puts $file [$w(faxnumberlist) get $i]
    }
    close $file
}

proc attachWindow {} { global fax c s w
    stat_text "Attach: Not implemented yet."
}

